import os
from typing import Any, Dict, List

from _jsonnet import evaluate_file

try:
    import ujson as json
    import json as vanilla_json
except ImportError:
    import json

    vanilla_json = json


def load_json(path: str) -> Dict[str, Any]:
    with open(path) as f:
        return json.load(f)


def write_json(result, path, pretty=True, encoder=None):
    if os.path.dirname(path).replace(".", ""):
        os.makedirs(os.path.dirname(path), exist_ok=True)
    with open(path, "w+") as f:
        if pretty:
            vanilla_json.dump(result, f, indent=4, separators=(',', ': '), cls=encoder)
        else:
            json.dump(result, f)


def load_jsonl(path: str) -> List[Dict[str, Any]]:
    with open(path) as f:
        return [json.loads(d) for d in f.read().splitlines()]


def write_jsonl(data: List[Dict[str, Any]], path, encoder=None):
    ds = [json.dumps(d, cls=encoder) for d in data]
    with open(path, 'w+') as f:
        f.write('\n'.join(ds) + '\n')


def load_jsonnet(path: str):
    return json.loads(evaluate_file(path))



from typing import Dict, Any
from handystuff.loaders import json


#
# try:
#     import click
# except ImportError:
#     raise ImportError("Install handystuff with prints options: \n\n pip install handystuff[prints]")
#
#
# def highlight(text, colors: Dict[str, str]):
#     result = []
#     for t in text.split():
#         if t in colors.keys():
#             result.append(click.style(t, fg=colors[t]))
#         else:
#             result.append(t)
#     return " ".join(result)


def format_dict(config: Dict[str, Any]) -> str:
    """Formats the config in a more-or-less readable way.

    Args:
      config: Dict[str:
      Any]:

    Returns:

    """
    return json.dumps(config, indent=4)



import math
from numbers import Real
from typing import List, Dict

try:
    import numpy as np
    from statsmodels.stats.proportion import proportion_confint
    from scipy.stats import t
except ImportError:
    raise ImportError("Install handystuff with stats options: \n\n pip install handystuff[stats]")


def get_mean_var_ci(sample, alpha=0.025):
    sample = np.array(sample)
    t_ci = t.ppf(1 - alpha, df=len(sample) - 1)
    return sample.mean(), sample.var(), t_ci * sample.std() / math.sqrt(len(sample))


def get_mean_var_ci_bernoulli(sample, alpha=0.05):
    if len(sample) > 0:
        lower, _ = proportion_confint(sum(sample), len(sample), alpha=alpha)
        mean = sum(sample) / len(sample)
        return mean, None, mean - lower
    else:
        return 0, None, 0


def mean_dict(result: List[Dict[str, Real]]) -> Dict[str, Real]:
    return {
        k: sum(r[k] for r in result) / len(result) for k in (result[0] if result else {}).keys()
    }



import sys

from setuptools import setup, find_packages

# Load requirements.txt
with open("requirements.txt") as requirements_file:
    lines = requirements_file.read().splitlines()
    install_requirements = []
    for line in lines:
        if '#egg' in line:
            name = line.split("#egg=")[-1]
            install_requirements.append(f"{name} @ {line}")
        else:
            install_requirements.append(line)

# make pytest-runner a conditional requirement,
# per: https://github.com/pytest-dev/pytest-runner#considerations
needs_pytest = {"pytest", "test", "ptr"}.intersection(sys.argv)
pytest_runner = ["pytest-runner"] if needs_pytest else []

setup_requirements = [
                         # add other setup requirements as necessary
                     ] + pytest_runner

setup(
    name='handystuff',
    version='0.0.3',
    description='Handy stuff that I end up copy-pasting from here and there',
    url='http://github.com/schlevik/handystuff',
    long_description=open("README.MD").read(),
    long_description_content_type="text/markdown",
    classifiers=[
        "Intended Audience :: Science/Research",
        "Development Status :: 3 - Alpha",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Programming Language :: Python :: 3.6",
        ],
    author='Viktor Schlegel',
    author_email='vtschlegel@gmail.com',
    license='GPLv3',
    packages=find_packages(exclude=["*.tests", "*.tests.*", "tests.*", "tests"]),
    install_requires=install_requirements,
    setup_requires=setup_requirements,
    tests_require=["pytest"],
    include_package_data=True,
    python_requires=">=3.7",
    package_data={"handystuff": ["resources/*"]},
    zip_safe=False,
    extras_require={
        'stats': ['statsmodels>=0.12', 'scipy>=1.6.0', 'numpy>=1.18'],
    }
)



import logging
from abc import ABC
from collections import Callable
from typing import TypeVar, Dict, Type

from handystuff.imports import construct

logger = logging.getLogger(__name__)

U = TypeVar("U")


class RegistryBase(ABC):
    """The factory class for creating executors"""

    """ Internal registry for available Algorithms """
    registry: Dict[str, U] = None

    @classmethod
    def get_name_of_instance(cls, registree: U) -> str:
        for k, v in cls.registry.items():
            if v == registree.__class__:
                return k
        raise ValueError(f"{registree.__class__.__name__} is not registered!")

    @classmethod
    def get_name_of_type(cls, registree: Type[U]) -> str:
        for k, v in cls.registry.items():
            if v == registree:
                return k
        raise ValueError(f"{registree.__name__} is not registered!")


    @classmethod
    def register(cls, name: str) -> Callable:
        def inner_wrapper(wrapped_class):
            if name in cls.registry:
                logger.warning('Registered item %s already exists. Will replace it', name)
            cls.registry[name] = wrapped_class
            return wrapped_class

        return inner_wrapper

    @classmethod
    def create(cls, name: str, args_as_dict=None, **kwargs):
        """Factory command to create the executor

        Args:
          name: str: 
          args_as_dict:  (Default value = None)
          **kwargs: 

        Returns:

        """

        class_instance = cls.registry.get(name, False)
        if not class_instance:
            raise ValueError(f"No implementation with name {name} exists. "
                             f"Did you forget to @Registry.register it?")
        else:
            object_instance = construct(class_instance, args_as_dict or {}, **kwargs)
            return object_instance
